<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.biuro.lt
 * @since      1.0.0
 *
 * @package    Biuro_Contacts
 * @subpackage Biuro_Contacts/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Biuro_Contacts
 * @subpackage Biuro_Contacts/public
 * @author     Biuro <info@biuro.lt>
 */
class Biuro_Contacts_Public {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    const FIELDS = [
        'employees' => ['name', 'phone', 'email', 'agree'],
        'employers' => ['name', 'phone', 'email', 'agree'],
        'position' => ['name', 'phone', 'email', 'agree', 'city', 'city_id', 'message', 'confirm', 'job_id'],
    ];

    const DB_NAMES = [
        'employees' => ['name', 'phone', 'email'],
        'employers' => ['name', 'phone', 'email'],
        'position' => ['name', 'phone', 'email', 'city', 'city_id', 'message', 'confirm', 'job_id']
    ];

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Biuro_Contacts_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Biuro_Contacts_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        // wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/biuro-contacts-public.css', array(), $this->version, 'all' );

    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Biuro_Contacts_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Biuro_Contacts_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        // wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/biuro-contacts-public.js', array( 'jquery' ), $this->version, false );

    }

    public static function insert_row_to_db($data, $format) {
        global $wpdb;

        $table = $wpdb->prefix . 'biuro_employees';

        $wpdb->insert( $table, $data, $format );

        return $wpdb->insert_id;
    }

    public static function update_row_in_db($data, $insert_id, $format) {
        global $wpdb;

        $table = $wpdb->prefix . 'biuro_employees';

        $where = array('id' => $insert_id);

        $where_format = array('%d');

        return $wpdb->update( $table, $data, $where, $format, $where_format );
    }


    public static function delete_sessions( $str ) {
        foreach (static::FIELDS[$str] as $key) {
           unset($_SESSION[$str . '--' . $key . '-message']);
           unset($_SESSION[$str . '--' . $key . '-status']);
           unset($_SESSION[$str . '--' . $key . '-value']);
        }
    }

    public static function validate( $str, $key, $value, $post ) {
        $empty = ['status' => '', 'message' => ''];
        $success = ['status' => 'success', 'message' => ''];
        $required = ['status' => 'error', 'message' => __('This field is required', 'biuro')];

        // if ($str == 'employees' || $str == 'position'):

        switch($key) {
        case 'name':

            return (!$value) ? $required : $success;

        case 'phone':
            if (!$value && !is_email($post['email'])):
                return [
                    'status' => 'error',
                    'message' => __('Phone or email field is required', 'biuro')
                ];
            endif;

            if (!$value):
                return $empty;
            endif;

            if ( strlen($value) < 6):
                return [
                    'status' => 'error',
                    'message' => __('Incorrect phone number', 'biuro')
                ];
            endif;

            return $success;

        case 'email':
            if (!$post['phone'] && !$value):
                return [
                    'status' => 'error',
                    'message' => __('Email or phone field is required', 'biuro')
                ];
            endif;

            if (!$post['phone'] && !is_email($value)):
                return [
                    'status' => 'error',
                    'message' => __('Email format is incorrect', 'biuro')
                ];
            else:

                if (!$value):
                    return $empty;
                endif;

                return $success;
            endif;

        case 'agree':
            if ($value != "1"):
                return [
                    'status' => 'error',
                    'message' => __('You have to agree with conditions', 'biuro')
                ];
            endif;

            return $success;

        case 'city':
        case 'city_id':
        case 'message':
        case 'confirm':
        case 'job_id':
            return $empty;
        }

        // endif;

        return $success;
    }

    public static function getValue( $key, $value ) {

        switch($key) {
        case 'name':
        case 'phone':
        case 'city':
            return sanitize_text_field($value);
        case 'email':
            return sanitize_email($value);
        case 'message':
            return sanitize_textarea_field($value);
        case 'cv':
            return sanitize_text_field($value);
        case 'agree':
        case 'city_id':
        case 'job_id':
        case 'confirm':
            return filter_var($value, FILTER_VALIDATE_INT) ? $value : 0;
        }

        return '';
    }

    public static function set_sessions( $str, $post ) {
        $canSubmit = true;

        $data = [
            'created' => current_time('Y-m-d H:i:s')
        ];

        foreach (static::FIELDS[$str] as $key) {
            $value = static::getValue($key, $post[$key]);
            $validation = static::validate($str, $key, $value, $post);

            if ($validation['status'] == 'error'):
                $canSubmit = false;
            endif;

            if ( in_array($key, static::DB_NAMES[$str])):
                $data[$key] = $value;
            endif;

            $_SESSION[$str . '--' . $key . '-value'] = $value;
            $_SESSION[$str . '--' . $key . '-status'] = $validation['status'];
            $_SESSION[$str . '--' . $key . '-message'] = $validation['message'];
        }

        if ($canSubmit):
            $insert_id = static::insert_row_to_db($data, array( '%s', '%s', '%s', '%s' ));

            $_SESSION[$str . '--step'] = 2;
            $_SESSION[$str . '--id'] = $insert_id;
        endif;
    }

    /**
     * Processes shortcode biuro-contacts--employees
     *
     * @param   array   $attr       The attributes from the shortcode
     *
     * @return  mixed   $output     Output of the buffer
     */
    public function employees_form( $attr = array() ) {
        ob_start();

        include_once('partials/biuro-contacts-public--employees.php');

        $output = ob_get_contents();

        ob_end_clean();

        static::delete_sessions('employees');

        return $output;
    } // employees_form()

    public function employees_post() {

        $nonce = $_POST['_wpnonce'];
        $referer = $_POST['_wp_http_referer'];


        if ( !isset( $nonce ) || !wp_verify_nonce($nonce, 'employees_post_nonce' ) ) {
            wp_redirect( $referer );
            exit;
        }

        unset($_SESSION['employees--step']);

        if ($_POST['submit'] == "1"):

            static::set_sessions('employees', $_POST);

        elseif ($_POST['submit'] == "2" && $_SESSION['employees--id']):

            $data = [
                'city' => static::getValue('city', $_POST['city']),
                'city_id' => static::getValue('city_id', $_POST['city_id']),
                'message' => static::getValue('message', $_POST['message']),
                'confirm' => static::getValue('confirm', $_POST['confirm']),
                'updated' => current_time('Y-m-d H:i:s')
            ];

            # upload file
            if($_FILES) {
                # gali nusirody: 'mimes' => array('csv' => 'text/csv')
                $movefile = wp_handle_upload($_FILES['cv'], [
                    'test_form' => false,
                    //'unique_filename_callback' => 'custom_file_name'
                ]);

                if ( $movefile && ! isset( $movefile['error'] ) ) {
                    $data['cv'] = static::getValue('cv', $movefile['url']);
                } else {
                    $_SESSION['employee_file_error'] = $movefile['error'];
                }
            }

            static::update_row_in_db($data, $_SESSION['employees--id'], array( '%s', '%d', '%s', '%d', '%s', '%s' ));

        endif;


        wp_redirect( $referer );
        exit;
    }

/* --------------------------- employers ---------------------------

    /**
     * Processes shortcode biuro-contacts--employers
     *
     * @param   array   $attr       The attributes from the shortcode
     *
     * @return  mixed   $output     Output of the buffer
     */
    public function employers_form( $attr = array() ) {
        ob_start();

        include_once('partials/biuro-contacts-public--employers.php');

        $output = ob_get_contents();

        ob_end_clean();

        static::delete_sessions('employers');

        return $output;
    } // employers_form()

    public function employers_post() {

        $nonce = $_POST['_wpnonce'];
        $referer = $_POST['_wp_http_referer'];


        if ( !isset( $nonce ) || !wp_verify_nonce($nonce, 'employers_post_nonce' ) ) {
            wp_redirect( $referer );
            exit;
        }

        unset($_SESSION['employers--step']);

        $str = 'employers';

        if ($_POST['submit'] == "1"):
            $canSubmit = true;

            $data = [
                'created' => current_time('Y-m-d H:i:s')
            ];

            foreach (static::FIELDS[$str] as $key) {
                $value = static::getValue($key,  $_POST[$key]);
                $validation = static::validate($str, $key, $value,  $_POST);

                if ($validation['status'] == 'error'):
                    $canSubmit = false;
                endif;

                if ( in_array($key, static::DB_NAMES[$str])):
                    $data[$key] = $value;
                endif;

                $_SESSION[$str . '--' . $key . '-value'] = $value;
                $_SESSION[$str . '--' . $key . '-status'] = $validation['status'];
                $_SESSION[$str . '--' . $key . '-message'] = $validation['message'];
            }

            if ($canSubmit):
                // $insert_id = static::insert_row_to_db($data, array( '%s', '%s', '%s', '%s' ));

                $_SESSION[$str . '--step'] = 2;
                $_SESSION[$str . '--id'] = $insert_id;
            endif;

            // static::set_sessions('employers', $_POST);

        elseif ($_POST['submit'] == "2"):

            $data = [
                'city' => static::getValue('city', $_POST['city']),
                'city_id' => static::getValue('city_id', $_POST['city_id']),
                'message' => static::getValue('message', $_POST['message']),
                'confirm' => static::getValue('confirm', $_POST['confirm']),
                'updated' => current_time('Y-m-d H:i:s')
            ];

            // # upload file
            // if($_FILES) {
            //     # gali nusirody: 'mimes' => array('csv' => 'text/csv')
            //     $movefile = wp_handle_upload($_FILES['cv'], [
            //         'test_form' => false,
            //         //'unique_filename_callback' => 'custom_file_name'
            //     ]);

            //     if ( $movefile && ! isset( $movefile['error'] ) ) {
            //         $data['cv'] = static::getValue('cv', $movefile['url']);
            //     } else {
            //         $_SESSION['employee_file_error'] = $movefile['error'];
            //     }
            // }

            // static::update_row_in_db($data, $_SESSION['employers--id'], array( '%s', '%d', '%s', '%d', '%s', '%s' ));

            debug($data);
            exit();
        endif;


        wp_redirect( $referer );
        exit;
    }

/* --------------------------- position ---------------------------

    /**
     * Processes shortcode biuro-contacts--position
     *
     * @param   array   $attr       The attributes from the shortcode
     *
     * @return  mixed   $output     Output of the buffer
     */
    public function position_form( $attr = array() ) {
        ob_start();

        if ($attr['job_id']):
            $job_id = $attr['job_id'];
        endif;

        include_once('partials/biuro-contacts-public--position.php');

        unset($_SESSION['position--step']);

        $output = ob_get_contents();

        ob_end_clean();

        static::delete_sessions('position');

        return $output;
    } // position_form()

    public function position_post() {

        $nonce = $_POST['_wpnonce'];
        $referer = $_POST['_wp_http_referer'];

        if ( !isset( $nonce ) || !wp_verify_nonce($nonce, 'position_post_nonce' ) ) {
            wp_redirect( $referer );
            exit;
        }

        // unset($_SESSION['position--step']);

        if ($_POST['submit'] == "1"):

            $_SESSION['position--step'] = 2;

        elseif ($_POST['submit'] == "2"):

            $_SESSION['position--step'] = 2;

            $canSubmit = true;

            $str = 'position';

            $data = [
                'created' => current_time('Y-m-d H:i:s'),
                'updated' => current_time('Y-m-d H:i:s')
            ];

            foreach (static::FIELDS[$str] as $key) {
                $value = static::getValue($key, $_POST[$key]);
                $validation = static::validate($str, $key, $value, $_POST);

                if ($validation['status'] == 'error'):
                    $canSubmit = false;
                endif;

                if ( in_array($key, static::DB_NAMES[$str])):
                    $data[$key] = $value;
                endif;

                $_SESSION[$str . '--' . $key . '-value'] = $value;
                $_SESSION[$str . '--' . $key . '-status'] = $validation['status'];
                $_SESSION[$str . '--' . $key . '-message'] = $validation['message'];
            }

            if ($canSubmit):

                unset($_SESSION['position--step']);

                # upload file
                if($_FILES) {
                    # gali nusirody: 'mimes' => array('csv' => 'text/csv')
                    $movefile = wp_handle_upload($_FILES['cv'], [
                        'test_form' => false,
                        //'unique_filename_callback' => 'custom_file_name'
                    ]);

                    if ( $movefile && ! isset( $movefile['error'] ) ) {
                        $data['cv'] = static::getValue('cv', $movefile['url']);
                    } else {
                        $_SESSION['position_file_error'] = $movefile['error'];
                    }
                }

                $insert_id = static::insert_row_to_db($data, array( '%s', '%s', '%s', '%s', '%s', '%s', '%d', '%s', '%d', '%d', '%s' ));

                $_SESSION['employee--id'] = $insert_id;

            endif;

        endif;


        wp_redirect( $referer );
        exit;
    }

    /**
    * @method getContacts
    * get contacts data from DB for API endpoint "contacts" endpoint;
    * @param WP_REST_Request $request
    * @return json;
    */
    public static function getContacts(WP_REST_Request $request)
    {
        global $wpdb;
        $params = $request->get_params();

        if ( !$params ) {
            exit;
        }

        $from = (string) $params['from'];
        $orderBy = (string) $params['by'];

        if(!$from || !in_array($orderBy, ['created', 'updated'])) {
            exit;
        }

        $sql = "SELECT * FROM `" . $wpdb->prefix . "biuro_employees`
            where ". $orderBy. " >= '". $from. "' ORDER BY ". $orderBy ." DESC LIMIT 50";
        return $wpdb->get_results($sql, ARRAY_A);
    }
}
